Package: gdalraster
Title: Bindings to 'GDAL'
Version: 2.6.1
Authors@R: c(
    person("Chris", "Toney", email = "jctoney@gmail.com",
            role = c("aut", "cre"), comment = c(ORCID = "0000-0001-5734-6510")),
    person("Michael D.","Sumner", role = c("aut"), comment = c(ORCID = "0000-0002-2471-7511")),
    person("Pepijn", "de Vries", role = c("ctb"), comment = c(ORCID = "0000-0002-7961-6646")),
    person("Frank", "Warmerdam",
            role = c("ctb", "cph"), comment = "GDAL API/documentation"),
    person("Even", "Rouault",
            role = c("ctb", "cph"), comment = "GDAL API/documentation"),
    person("Marius", "Appel",
            role = c("ctb", "cph"), comment = "configure.ac based on https://github.com/appelmar/gdalcubes"),
    person("Daniel", "James",
            role = c("ctb", "cph"), comment = "Boost combine hashes method in src/cmb_table.h"),
    person("Peter", "Dimov",
            role = c("ctb", "cph"), comment = "Boost combine hashes method in src/cmb_table.h"))
Description: API bindings to the Geospatial Data Abstraction Library ('GDAL',
    <https://gdal.org>). Implements the 'GDAL' Raster and Vector Data Models.
    Bindings are implemented with 'Rcpp' modules. Exposed C++ classes and
    stand-alone functions wrap much of the 'GDAL' API and provide additional
    functionality. Calling signatures resemble the native C, C++ and Python APIs
    provided by the 'GDAL' project. Class 'GDALRaster' encapsulates a
    'GDALDataset' and its raster band objects. Class 'GDALVector' encapsulates
    an 'OGRLayer' and the 'GDALDataset' that contains it. Initial bindings are
    provided to the unified 'gdal' command line interface added in 'GDAL' 3.11.
    C++ stand-alone functions provide bindings to most 'GDAL' "traditional"
    raster and vector utilities, including 'OGR' facilities for vector
    geoprocessing, several algorithms, as well as the Geometry API ('GEOS' via
    'GDAL' headers), the Spatial Reference Systems API, and methods for
    coordinate transformation. Bindings to the Virtual Systems Interface ('VSI')
    API implement standard file system operations abstracted for URLs, cloud
    storage services, 'Zip'/'GZip'/'7z'/'RAR', in-memory files, as well as
    regular local file systems. This provides a single interface for operating
    on file system objects that works the same for any storage backend. A custom
    raster calculator evaluates a user-defined R expression on a layer or stack
    of layers, with pixel x/y available as variables in the expression. Raster
    'combine()' identifies and counts unique pixel combinations across multiple
    input layers, with optional raster output of the pixel-level combination
    IDs. Basic plotting capability is provided for raster and vector display.
    'gdalraster' leans toward minimalism and the use of simple, lightweight
    objects for holding raw data. Currently, only minimal S3 class interfaces
    have been implemented for selected R objects that contain spatial data.
    'gdalraster' may be useful in applications that need scalable, low-level
    I/O, or prefer a direct 'GDAL' API.
License: MIT + file LICENSE
Copyright: See file inst/COPYRIGHTS for details.
URL: https://firelab.github.io/gdalraster/,
        https://github.com/firelab/gdalraster
BugReports: https://github.com/firelab/gdalraster/issues
Depends: R (>= 4.2.0)
Imports: bit64, cli, graphics, grDevices, methods, nanoarrow, Rcpp (>=
        1.0.7), stats, tools, utils, wk, xml2, yyjsonr
LinkingTo: cli, nanoarrow, Rcpp, RcppInt64
Suggests: glue (>= 1.6.0), gt, knitr, ltc, rmarkdown, scales, testthat
        (>= 3.0.0), vctrs (>= 0.3.0)
NeedsCompilation: yes
SystemRequirements: C++17, GDAL (>= 3.1.0, built against GEOS)
Encoding: UTF-8
VignetteBuilder: knitr
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
Packaged: 2026-05-16 00:40:47 UTC; ctoney
Author: Chris Toney [aut, cre] (ORCID: <https://orcid.org/0000-0001-5734-6510>),
  Michael D. Sumner [aut] (ORCID:
    <https://orcid.org/0000-0002-2471-7511>),
  Pepijn de Vries [ctb] (ORCID: <https://orcid.org/0000-0002-7961-6646>),
  Frank Warmerdam [ctb, cph] (GDAL API/documentation),
  Even Rouault [ctb, cph] (GDAL API/documentation),
  Marius Appel [ctb, cph] (configure.ac based on
    https://github.com/appelmar/gdalcubes),
  Daniel James [ctb, cph] (Boost combine hashes method in
    src/cmb_table.h),
  Peter Dimov [ctb, cph] (Boost combine hashes method in src/cmb_table.h)
Maintainer: Chris Toney <jctoney@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-16 11:50:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-05-16 12:21:58 UTC; unix
Archs: gdalraster.so.dSYM
