useDynLib(gdalraster, .registration=TRUE)
import(bit64, graphics, methods, nanoarrow, Rcpp, wk, xml2)
importFrom("cli", "cli_progress_bar", "cli_progress_update", "cli_progress_done")
importFrom("grDevices", "as.raster", "colorRamp", "dev.capabilities", "gray", "rgb")
importFrom("stats", "quantile")
importFrom("tools", "file_path_sans_ext")
importFrom("utils", "file_test", "setTxtProgressBar", "txtProgressBar")
importFrom("yyjsonr", "read_json_str")
exportPattern("^[[:alpha:]]+")
S3method(plot, OGRFeature)
S3method(plot, OGRFeatureSet)
S3method(print, OGRFeature)
S3method(print, OGRFeatureSet)
